/*******************************************************************************
DeskCalc. Copyright 1997-2000 Wunderbear Software. See below for license info.
$Id: CalcView.h,v 1.8 1998/12/03 08:29:39 timmy Exp $
Code Classification: C++, BeOS

This component implements actual functional GUI interface of the calculator.

Copyright (c) 1997-2000 Wunderbear Software. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1) Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2) Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

3) Neither the name of the Wunderbear Software nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COMPANY OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/


#ifndef _CALC_VIEW_H
#define _CALC_VIEW_H


// BeOS Includes

#ifndef _RECT_H
	#include <interface/Rect.h>
#endif

#ifndef _VIEW_H
	#include <interface/View.h>
#endif

#ifndef _MESSAGE_H
	#include <app/Message.h>
#endif

#ifndef _GRAPHICS_DEFS_H
	#include <interface/GraphicsDefs.h>
#endif

#ifndef _MENU_ITEM_H
	#include <interface/MenuItem.h>
#endif


// Class Definitions

struct CalcKey {
	
	char label[8];
	char code[8];
	char keymap[4];
	uint32 flags;
	float width;
	
}; // end struct CalcKey


class BString;
class CalcOptions;
class CalcOptionsWindow;

_EXPORT class CalcView : public BView {

	public:
		
		// ****** STATICS ******
		
		static CalcView *Instantiate(BMessage *archive);
		
		
		// ****** CREATORS ******
		
		// Constructor.
		CalcView(BRect frame,
			const char *calcDesc,
			rgb_color rgbBaseColor);

		// Construct from archive.
		CalcView(BMessage *archive);

		// Destructor.
		virtual ~CalcView();

		
		// ****** MANIPULATORS ******
		
		// Message received hook.
		virtual void MessageReceived(BMessage *message);
		
		// Render view hook.
		virtual void Draw(BRect updateRect);
		
		// Mouse button pressed hook.
		virtual void MouseDown(BPoint point);
		
		// Key depressed hook.
		virtual void KeyDown(const char *bytes, int32 numBytes);
		
		// View receives focus hook.
		virtual void MakeFocus(bool focused = true);
		
		
		// ****** ACCESSORS ******
		
		// Present about box for view (replicant).
		virtual void AboutRequested();
		
		// Archive this view.
		virtual status_t Archive(BMessage *archive, bool deep) const;
		
		// Cut contents of view to system clipboard.
		void Cut();
		
		// Copy contents of view to system clipboard.
		void Copy();
		
		// Paste contents of system clipboard to view.
		void Paste(BMessage *message);
		
	
	private:
		
		// ****** CONSTANTS ******
		enum {
			K_OPTIONS_REQUESTED =	'opts'
			}; // end enum
		
		// Parse calculator description for view.
		void ParseCalcDesc(const char *calcDesc);
		
		// Press a key, any key.
		void PressKey(int key);
		
		// Press a key by label.
		void PressKey(char *label);
		
		// Sort out the color scheme.
		void Colorize();
		
		// Evaluate current expression.
		void Evaluate();
		
		// Create pop-up menu system.
		void CreatePopUpMenu();

		int16 mCols, mRows;			// grid dimensions
		rgb_color mRGBBaseColor;	// color scheme
		rgb_color mRGBLightEdge;
		rgb_color mRGBDarkEdge;
		rgb_color mRGBDisplay;
		float mWidth, mHeight;		// view dimensions
		char *mCalcDesc;			// keypad grid
		CalcKey *mKeypad;
		BString *mDisplayText;		// display text
		BMenuItem *mAboutItem;		// pop-up context menu.
		BMenuItem *mOptionsItem;
		BPopUpMenu *mPopUpMenu;
		CalcOptions *mOptions;		// calculator options.
		CalcOptionsWindow *mOptionsWindow;
		
}; // end class CalcView


/******************************************************************************/

#endif // _CALC_VIEW_H

// end CalcView.h
